#
# Copyright (c) 2021-2024 ahriman team.
#
# This file is part of ahriman
# (see https://github.com/arcan1s/ahriman).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
from dataclasses import dataclass, field
from pathlib import Path

from ahriman.core.util import trim_package


@dataclass(frozen=True, kw_only=True)
class FilesystemPackage:
    """
    class representing a simplified model for the package installed to filesystem

    Attributes:
        package_name(str): package name
        depends(set[str]): list of package dependencies
        directories(set[Path]): list of directories this package contains
        files(list[Path]): list of files this package contains
        opt_depends(set[str]): list of package optional dependencies
    """

    package_name: str
    depends: set[str]
    opt_depends: set[str]
    directories: list[Path] = field(default_factory=list)
    files: list[Path] = field(default_factory=list)

    def __post_init__(self) -> None:
        """
        update dependencies list accordingly
        """
        object.__setattr__(self, "depends", {trim_package(package) for package in self.depends})
        object.__setattr__(self, "opt_depends", {trim_package(package) for package in self.opt_depends})

    def depends_on(self, package_name: str) -> bool:
        """
        check if package depends on given package name

        Args:
            package_name(str): package name to check dependencies

        Returns:
            bool: True in case if the given package in the dependencies or optional dependencies lists
        """
        return package_name in self.depends or package_name in self.opt_depends

    def __repr__(self) -> str:
        """
        generate string representation of object

        Returns:
            str: unique string representation
        """
        return f'FilesystemPackage(package_name={self.package_name}, depends={self.depends})'
